#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 19-Sep-2003
#################################################################################
#
# Bio - Script part of Ethereal Realms designed to show a puppet/puppeteer bio 
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Comm;						# Communication handler
use eRealms::Database;						# Database handler
use eRealms::Login;						# Login handler
use eRealms::Param;						# Parameter control
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $comm;							# Communications panel
my $database;							# Database handle
my $login;							# Login display
my $param;							# Parameter handle
my $tmpl;							# Template handle

my $base;							# Base document
my $contact;							# Contacts
my $exist;							# Existence
my $nav;							# Navigational variable
my $panel;							# Comm panel widget

my $bio;							# Bio Link
my $display;							# Display information
my $user;							# Username

my %realm;							# Configuration hash


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and Fetch
	$database->Connect();

	# Parameter handling
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Link with hash
	$database->GetHashRealm(\%realm);

	
	# Initialize handles
	$tmpl     = new eRealms::Template(\%realm);


	# Pull information
	# Base template
	($base) = $database->DocumentGetBioBase();

	# Puppet specific homepage
	($bio, $display, $user) = $database->DataGet("SELECT
		 PuppetBio,
		 PuppetDisplay,
		 PuppeteerLogin
		FROM  Puppet
		WHERE PuppetName=?
		  AND PuppetPool=?", 
	
	 $param->{'BCHAR'},
	 $param->{'BPOOL'}) if (defined($param->{'BCHAR'}));

	# Must be defined
	$bio = (defined($bio)) ? $bio : $realm{'LnkHome'};


	# Authenthicated users
	# Advanced functions
	if (defined($param->{'USER'}))
	{
		# Login and template instance
		$login = new eRealms::Login($database, $cgi, $param);
		
		# Authenthication
		if ($login->GetVerificationRealm())
		{
			# Prevent bans
			$login->CheckLegality(\%realm);

			# Contact information
			if (defined($param->{'CONTACT'}))
			{
				# Fetch navigational structure
				$nav = CreateNav($database, $cgi, $param, $tmpl, \%realm, $bio);

				# Contact information
				$contact = CreateContact($database, $param, \%realm, $user, $display);

				# Search and replace
				$base =~ s/MCHAR/$param->{'BCHAR'}/gs;
				$base =~ s/MMAIN/$contact/gs;
				$base =~ s/MMOVE/$nav/gs;


				# Document display
				print $cgi->header();
				print $base;
			}

			# Puppet level ignore
			if (defined($param->{'IGGY'}))
			{
				# Insert ignore
				$database->Quick("INSERT INTO PuppetIgnore
					(PuppeteerLogin,
					 PuppetName,
					 PuppetIgnore)
					VALUES(?,?,'single')",

				 $param->{'USER'},
				 $param->{'BCHAR'}
				);


				# Fetch navigational structure
				$nav = CreateNav($database, $cgi, $param, $tmpl, \%realm, $bio);

				# Contact information
				$contact = CreateContact($database, $param, \%realm, $user, $display);

				# Search and replace
				$base =~ s/MCHAR/$param->{'BCHAR'}/gs;
				$base =~ s/MMAIN/$contact/gs;
				$base =~ s/MMOVE/$nav/gs;


				# Document display
				print $cgi->header();
				print $base;
			}

			# Private messaging
			else
			{
				# Create instance
				$comm  = new eRealms::Comm($database, $cgi);


				# Make sure it's valid
				if (defined($param->{'CHAR'}))
				{
					# Existence
					($exist) = $database->DataGetPuppetExistence($param->{'CHAR'}, $param->{'USER'}) ;

					# Invalid handle
					unless (defined($exist))
					{
						# Ensure definition
						($param->{'CHAR'}) = $database->DataGetDefault($param->{'USER'});
					}
				}
				else
				{
					# Ensure definition
					($param->{'CHAR'}) = $database->DataGetDefault($param->{'USER'});
				}


				# Writing of post
				if (defined($param->{'PSEND'}))
				{
					# Create variables
					$param->{'SOURCE'}  = $param->{'USER'};
					$param->{'TARGET'}  = $user;
					$param->{'OTARGET'} = $param->{'BCHAR'};

					# Write post
					$comm->DataWrite($param);

					# Clean up
					$param->Cleanup('SOURCE', 'TARGET', 'OTARGET');
				}


				# Fetch navigational structure
				$nav = CreateNav($database, $cgi, $param, $tmpl, \%realm, $bio);

				# Contact information
				$panel = CreatePanel($database, $cgi, $param, $comm, \%realm);

				# Search and replace
				$base =~ s/MCHAR/$param->{'BCHAR'}/gs;
				$base =~ s/MMAIN/$panel/gs;
				$base =~ s/MMOVE/$nav/gs;


				# Document display
				print $cgi->header();
				print $base;
			}
		}
	}

	# Limited functions
	else
	{
		# Enough information to handle
		if (defined($display))
		{
			# Straight redirect
			if ($display eq 'no')
			{
				# Create redirect header
				print $cgi->redirect($bio);
			}

			# Normal operation
			else
			{
				# Fetch navigational structure
				$nav = CreateNav($database, $cgi, $param, $tmpl, \%realm, $bio);

				# Contact information
				$contact = CreateContact($database, $param, \%realm, $user, $display);

				# Search and replace
				$base =~ s/MCHAR/$param->{'BCHAR'}/gs;
				$base =~ s/MMAIN/$contact/gs;
				$base =~ s/MMOVE/$nav/gs;


				# Document display
				print $cgi->header();
				print $base;
			}
		}

		# Warn user
		else
		{
			$contact = ""
			  . " <BR>\n"
			  . "  <TABLE WIDTH=\"100%\">\n"
			  . "   <TR>\n"
			  . "    <TD>$realm{'MsgContact'}</TD>\n"
			  . "   </TR>\n"
			  . "  </TABLE>\n";


			# Search and replace
			$base =~ s/MCHAR/$param->{'BCHAR'}/gs;
			$base =~ s/MMAIN/$contact/gs;
			$base =~ s/MMOVE/$nav/gs;

			# Document header
			print $cgi->header();
			print $base;
		}
	}


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Create Contact
#
# Will generate the contact addresses for users and return it to be implemented
# into a greater scheme.

sub CreateContact
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list
	my $realm    = shift;					# Realm hash
	my $user     = shift;					# Users login name
	my $display  = shift;					# Permission to display

	my $count;						# What is available
	my $type;						# Type of contact
	my $contact;						# Formatted entry

	my $res;						# Results hash
	my $statement;						# Database query


	#####################
	# Program Area

	# Determine what is available
	($count) = $database->DataGet("SELECT COUNT(*)
		FROM PuppeteerContact
		WHERE PuppeteerLogin=?", $user);


	# Initial assignment
	$contact =  "";

	# Verify need
	# Create display
	if (($display eq 'yes')
	 && ($count > 0))
	{ 
		# Reset counter
		$count = 0;


		# Header
		$contact = $contact
		  . "  <BR>\n"
		  . "  <TABLE WIDTH=\"100%\">\n"
		  . "   <TR>\n"
		  . "    <TD>\n";

		# Construction
		# Display images
		$database->Pull(\$statement, "SELECT 
			  c.ContactName   AS \"Name\", 
			  d.ContactData   AS \"Data\"
			FROM 
			  Contact c,
			  PuppeteerContact d
			WHERE d.ContactName = c.ContactName
			  AND c.ContactType=?
			  AND d.PuppeteerLogin=?
			ORDER BY c.ContactType, c.ContactName",

		'picture', $user);

		# Create all
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			# Buffer
			$contact = $contact . "     <BR><BR>\n" if ($count > 1);

			# Image creation
			$contact = $contact . "     <IMG SRC=\"$res->{Data}\" ALT=\"$res->{Data}\">\n\n";


			# Increment to keep track
			$count++;
		}

		# End query
		$statement->finish();


		# Additional spacer
		$contact = $contact . "      &nbsp;\n" if ($count < 1);;


		# Seperator
		$contact = $contact
		  . "    </TD>\n"
		  . "    <TD>\n"
		  . "     <TABLE WIDTH=\"100%\">\n";



		# Run through and display
		$database->Pull(\$statement, "SELECT 
			  c.ContactName   AS \"Name\", 
			  c.ContactFilter AS \"Filter\", 
			  c.ContactType   AS \"Type\", 
			  d.ContactData   AS \"Data\"
			FROM 
			  Contact c,
			  PuppeteerContact d
			WHERE d.ContactName = c.ContactName
			  AND c.ContactType <> ?
			  AND d.PuppeteerLogin=?
			ORDER BY c.ContactType, c.ContactName",

		'picture', $user);


		# Initial retrieval
		$res  = $statement->fetchrow_hashref();
		$type = 'None';



		# Single run
		# Type handle
		$type = CheckType(\$contact, $res->{'Type'}, $type, $realm->{'TmplContact'});

		# Display row
		ContentDisplay(\$contact, $res->{'Filter'}, $res->{'Type'}, $res->{'Name'}, $res->{'Data'});


		# Possibly loop
		while ($res = $statement->fetchrow_hashref())
		{
			# Looped operation
			# Type handler
			$type = CheckType(\$contact, $res->{'Type'}, $type, $realm->{'TmplContact'});

			# Display row
			ContentDisplay(\$contact, $res->{'Filter'}, $res->{'Type'}, $res->{'Name'}, $res->{'Data'});
		}

		# End query
		$statement->finish();

		# Close up
		$contact = $contact
		  . "     </TABLE>\n"
		  . "    </TD>\n"
		  . "   </TR>\n"
		  . "  </TABLE>\n";
	}

	# Default messages
	else
	{
		$contact = $contact
		  . " <BR>\n"
		  . "  <TABLE WIDTH=\"100%\">\n"
		  . "   <TR>\n"
		  . "    <TD>$realm{'MsgContact'}</TD>\n"
		  . "   </TR>\n"
		  . "  </TABLE>\n";
	}


	# Returns generated content
	return $contact;
}


#####################
# Create Nav
#
# Creates the navigational menu system necessary to be able to move from
# one component to the other

sub CreateNav
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handling
	my $realm    = shift;					# Reference to realm hash
	my $bio      = shift;					# Biographical link

	my $nav;						# Navigational string
	my $ignore;
	my $url  = $cgi->url(-relative=>1);;			# Self referencing link

	my %args;						# Cut down on repetitive arguments

	#####################
	# Program area

	# Initial assignment
	$nav = $tmpl->Pass('TagBioNav', MLINK=>"$bio\" TARGET=\"_BLANK", MNAME=>$realm->{'TagBioHome'});


	# Initial parameters
	%args = (
		BCHAR => $param->{'BCHAR'},
		BPOOL => $param->{'BPOOL'}
	);


	# Verified if user is logged on
	# Not the case
	unless (defined($param->{'USER'}))
	{
		# Generate inline
		my $inline = $param->Crypt(%args);

		# Contact Information
		$nav = $nav . $tmpl->Pass('TagBioNav', MLINK=>"$url$inline", MNAME=>$realm->{'TagBioContact'});
	}

	# Is in case
	else
	{
		# Verify if ignored
		($ignore) = $database->DataGet("SELECT PuppetName
			FROM PuppetIgnore
			WHERE PuppetName=?
			  AND PuppeteerLogin=?",

		 $param->{'BCHAR'},
		 $param->{'USER'}
		);
			

		# Additional parameters
		$args{'USER'}  = $param->{'USER'};
		$args{'CRYPT'} = $param->{'CRYPT'};
		$args{'ROOM' } = $param->{'ROOM'};

		# Generate inlines
		my $ictc = $param->EmbedInline(%args, CONTACT=>'True');
		my $iggy = $param->EmbedInline(%args, IGGY=>'True');
		my $ipst = $param->EmbedInline(%args, );
		
		
		# Contact Information
		$nav = $nav . $tmpl->Pass('TagBioNav', MLINK=>"$url$ictc", MNAME=>$realm->{'TagBioContact'});
		$nav = $nav . $tmpl->Pass('TagBioNav', MLINK=>"$url$iggy", MNAME=>$realm->{'TagBioIgnore'}) unless (defined($ignore));
		$nav = $nav . $tmpl->Pass('TagBioNav', MLINK=>"$url$ipst", MNAME=>$realm->{'TagBioPrivate'});
	}

	# Return generated code
	return $nav;
}


####################
# Panel Creation
#
# Display prompt for a private message, this includes a list of all the puppets
# et cetera. 

sub CreatePanel
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Cgi handle
	my $param    = shift;					# Parameter handle
	my $comm     = shift;					# Communications panel
	my $realm    = shift;					# Realm options

	my $option;						# Option handle
	my $who;						# Who is on handle

	my $access;						# Access control
	my $defsize; 						# Default size
	my $defcolor;						# Default colour
	my $defface;						# Default face

	my $panel;						# Private message panel

	my @char;						# List of characters
	my @color;						# Colour choices
	my @face;						# Font face

	my %who;						# Associative hash
	

	####################
	# Program area

	# Create instances
	$option   = new eRealms::Option();


	# Pull values
	($access) = $database->DataGetRealmAccess($param->{'ROOM'});


	# Populate arrays
        @color = $comm->FontSelectColor($param);
        @face  = $comm->FontSelectFace($param);

	# Populate puppet list
	$database->GroupList(\@char, $access, $param->{'USER'}, $param->{'ROOM'}); 

	
	# Retreive size and color
	($defsize, $defcolor, $defface) = $database->DataGetFont($param->{'CHAR'}, $param->{'USER'}, $color[0], $face[0]);

	# Cleanup for embed
	$param->Cleanup('POST');


	# Retreive private message panel
	$panel = $database->DocumentGetBioPrivate();


	# Widgets
	# Assigning values
	my $wpostsend = $cgi->submit(
		-name     => $realm->{'TxtSubmitButton'},
		-value    => $realm->{'TxtSubmitButton'},
		-tabindex => 2);

	# Posting window
	my $wposttext = $cgi->textarea(
		-name     => 'POST',
		-rows     => 3,
		-columns  => 55,
		-wrap     => 'SOFT',
		-tabindex => 1);

	# Selection
	# Our own
	my $wcharslct = $cgi->popup_menu(
		-name     => 'CHAR',
		-values   => \@char,
		-default  => $param->{'CHAR'},
		-tabindex => 3);

	# Form values
	my $wformstrt = $cgi->start_form();
	my $wformhidn = $param->EmbedNormal($param->Flat(),
		 SIZE  => $defsize,
		 COLOR => $defcolor,
		 FACE  => $defface,
		 PSEND => 'True');
	my $wformends = $cgi->end_form();


	# Widget replacements
	# Posting widgets
	$panel =~ s/WPOSTSEND/$wpostsend/gs;
	$panel =~ s/WPOSTTEXT/$wposttext/gs;

	# Puppet selection
	$panel =~ s/WCHARSLCT/$wcharslct/gs;

	# Form
	$panel =~ s/WFORMSTRT/$wformstrt$wformhidn/gs;
	$panel =~ s/WFORMENDS/$wformends/gs;


	# Document
	return $panel;
}


#####################
# Type checks
#
# Will display titles if need be.   This is needed since the loop is a bit
# different then with other Contact scripts.

sub CheckType
{
	#####################
	# Data members
	my $pass   = shift;					# Passed string ref
	my $check  = shift;					# Value to check
	my $type   = shift;					# Mainstay type
	my $title  = shift;					# Title to display


	#####################
	# Program area

	# Type display
	# Ensure assignment
	$check = (defined($check)) ? $check : 'None';

	# Determine need
	if (ucfirst($check) ne $type)
	{
		# Assign type
		$type = ucfirst($check);

		# Set title
		$title =~ s/MTITLE/$type/gs;

		# Display
		${$pass} = ${$pass}
		  . "      <TR>\n"
		  . "       <TD COLSPAN=\"2\" HEIGH=\"15\">&nbsp;</TD>\n"
		  . "      </TR>\n"
		  . "      <TR>\n"
		  . "       <TD COLSPAN=\"2\">$title</TD>\n"
		  . "      </TR>\n";
	}

	# Return
	return $type;
}


#####################
# Content display
#
# Will display the contents of the variables.  Once it determines if there
# is a need to create links or some such.

sub ContentDisplay
{
	#####################
	# Data members
	my $pass   = shift;					# Passed string ref
	my $filter = shift;					# Filter rule
	my $type   = shift;					# Type of information
	my $name   = shift;					# Name of info
	my $data   = shift;					# Information to process


	#####################
	# Program area

	# Email address
	if ($filter eq 'email')
	{
		# Create mail link
		${$pass} = ${$pass}
		  . "      <TR>\n"
		  . "       <TD CLASS=\"BIOTIT\">$name</TD>\n"
		  . "       <TD CLASS=\"BIORES\"><A HREF=\"mailto:$data\">$data</A></TD>\n"
		  . "      </TR>\n";
	}

	# Standard URL
	elsif ($filter eq 'http')
	{
		${$pass} = ${$pass}
		  . "      <TR>\n"
		  . "       <TD CLASS=\"BIOTIT\">$name</TD>\n"
		  . "       <TD CLASS=\"BIORES\"><A HREF=\"$data\" TARGET=\"_new\">$data</A></TD>\n"
		  . "      </TR>\n";
	}

	# All other types
	else
	{
		${$pass} = ${$pass}
		  . "      <TR>\n"
		  . "       <TD CLASS=\"BIOTIT\">$name</TD>\n"
		  . "       <TD CLASS=\"BIORES\">$data</TD>\n"
		  . "      </TR>\n";
	}
}
